package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import cn.com.qmth.stmms.biz.config.service.impl.SystemCache;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.file.enums.FormatType;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.report.model.ReportSubject;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectClass;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectCollege;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectGroup;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectTeacher;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectClassService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectCollegeService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectGroupService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectTeacherService;
import cn.com.qmth.stmms.biz.report.utils.ReportContext;
import cn.com.qmth.stmms.biz.report.utils.ReportSubjectRangeDTO;
import cn.com.qmth.stmms.biz.utils.ExportExcel;
import cn.com.qmth.stmms.biz.utils.TaskLockUtil;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;

import com.qmth.boot.tools.codec.CodecUtils;

/**
 * 与评卷相关的所有定时任务
 *
 * @author luoshi
 */
@Component
public class MarkCronService {

    protected static final Logger log = LoggerFactory.getLogger(MarkCronService.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkService markService;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private SystemCache systemCache;

    @Autowired
    private ExamService examService;

    @Value("${mark.activeExpireMinute}")
    private long markerActiveExpireMinute;

    @Autowired
    private FileService fileService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private ReportService reportService;

    @Autowired
    private ReportSubjectQuestionService reportSubjectQuestionService;

    @Autowired
    private ReportSubjectClassService reportSubjectClassService;

    @Autowired
    private ReportSubjectTeacherService reportSubjectTeacherService;

    @Autowired
    private ReportSubjectCollegeService reportSubjectCollegeService;

    @Autowired
    private ReportSubjectGroupService reportSubjectGroupService;

    @Autowired
    private ReportSubjectService reportSubjectService;

    private ReportContext context;

    private Map<String, List<ExamQuestion>> objectiveMap;

    private Map<String, List<ExamQuestion>> subjectiveMap;

    private Map<String, ExamSubject> subjectMap;

    /**
     * 自动释放可清除的锁
     */
    @Scheduled(cron = "${mark.cleanLockSchedule}")
    public void cronCleanLock() {
        try {
            lockService.clear();
        } catch (Exception e) {
            log.error("CronCleanTask error", e);
        }
    }

    /**
     * 自动释放超时未处理的评卷任务
     */
    @Scheduled(cron = "${mark.cleanTaskSchedule}")
    public void cronCleanTask() {
        try {
            long timeoutMinute = systemCache.getMarkTime();
            TaskLockUtil.clearTimeoutTask(timeoutMinute * 60 * 1000);
        } catch (Exception e) {
            log.error("CronCleanTask error", e);
        }
    }

    /**
     * 定时生成评卷任务
     */
    @Scheduled(fixedDelay = 60 * 1000, initialDelay = 60 * 1000)
    public void buildLibrary() {
        log.info("start auto-create library");
        try {
            List<Integer> examIds = groupService.findExamIdByStatus(MarkStatus.TRIAL, MarkStatus.FORMAL);
            for (Integer examId : examIds) {
                Exam exam = examService.findById(examId);
                if (ExamStatus.FINISH.equals(exam.getStatus())) {
                    continue;
                }
                buildLibraryByExam(examId);
            }
        } catch (Exception e) {
            log.error("auto-create library error", e);
        } finally {
            log.info("finish auto-create library");
        }
    }

    /**
     * 定时更新评卷员质量监控指标
     */
    @Scheduled(fixedDelay = 60 * 1000, initialDelay = 60 * 1000)
    public void updateMarkerQuality() {
        log.info("start auto-update marker");
        try {
            List<Integer> examIds = groupService.findExamIdByStatus(MarkStatus.FORMAL);
            for (Integer examId : examIds) {
                updateMarkerQualityByExam(examId);
            }
        } catch (Exception e) {
            log.error("auto-update marker error", e);
        } finally {
            log.info("finish auto-update marker");
        }
    }

    private void updateMarkerQualityByExam(Integer examId) {
        // 获取主观题总分大于0的科目
        List<ExamSubject> subjects = subjectService.list(examId, 0);
        for (ExamSubject subject : subjects) {
            List<MarkGroup> groups = groupService.findByExamAndSubjectAndStatus(examId, subject.getCode(),
                    MarkStatus.FORMAL);
            for (MarkGroup markGroup : groups) {
                List<Marker> markers = markerService.findByExamAndSubjectAndGroup(examId, subject.getCode(),
                        markGroup.getNumber());
                for (Marker marker : markers) {
                    try {
                        if (markService.needUpdateQuality(marker, markerActiveExpireMinute)) {
                            log.info("start auto-update markerId: " + marker.getId());
                            markService.updateQuality(marker);
                        }
                    } catch (Exception e) {
                        log.error("update marker quality error for markerId=" + marker.getId(), e);
                    }
                }
            }
        }
    }

    private void buildLibraryByExam(Integer examId) {
        // 获取主观题总分大于0的科目
        List<ExamSubject> subjects = subjectService.listSubjectiveScore(examId, 0d);
        for (ExamSubject subject : subjects) {
            // 清除缺考考生和违纪考生
            List<ExamStudent> list = studentService.findAbsentOrBreachLibraryStudent(subject.getExamId(),
                    subject.getCode());
            if (list != null) {
                for (ExamStudent student : list) {
                    try {
                        lockService.waitlock(LockType.STUDENT, student.getId());
                        markService.deleteByStudent(student);
                    } catch (Exception e) {
                        log.error("delete student library error", e);
                    } finally {
                        lockService.unlock(LockType.STUDENT, student.getId());
                    }
                }
            }
            // 处理正常考生
            // 生成试评任务
            List<MarkGroup> groups = groupService.findByExamAndSubjectAndStatus(subject.getExamId(), subject.getCode(),
                    MarkStatus.TRIAL);
            long trialCount = studentService.countByExamIdAndSubjectCodeAndStatus(examId, subject.getCode(),
                    SubjectiveStatus.TRIAL, SubjectiveStatus.MARKED, SubjectiveStatus.INSPECTED);
            if (!groups.isEmpty() && trialCount < subject.getTrialCount()) {
                while (trialCount < subject.getTrialCount()) {
                    // 取一个的考生
                    ExamStudent student = studentService.randomStudent(examId, subject.getCode());
                    if (student == null) {
                        break;
                    }
                    studentService.updateSubjectiveStatusAndTimeAndInspectorId(student.getId(), SubjectiveStatus.TRIAL,
                            null, null);
                    trialCount++;
                }
            }
            for (MarkGroup group : groups) {
                buildTrialLibrary(trialCount, group);
            }
            // 生成正评任务
            // 试评状态的分组也提前生成
            groups = groupService.findByExamAndSubjectAndStatus(subject.getExamId(), subject.getCode(),
                    MarkStatus.FORMAL, MarkStatus.TRIAL);
            for (MarkGroup group : groups) {
                buildFormalLibrary(subject, group);
            }
        }
    }

    private void buildFormalLibrary(ExamSubject subject, MarkGroup group) {
        Date lastBuildTime = group.getBuildTime();
        int count = 0;
        ExamStudent student = studentService.findUnLibraryStudent(group.getExamId(), group.getSubjectCode(),
                group.getNumber(), lastBuildTime);
        while (student != null) {
            // 尝试构造评卷任务
            try {
                lockService.watch(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
                lockService.watch(LockType.STUDENT, student.getId());
                // 上锁后重复验证分组状态
                if (!groupService.validateStatus(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                        MarkStatus.FORMAL, MarkStatus.TRIAL)) {
                    break;
                }
                // 上锁后重复验证考生状态
                if (studentService.validateStatus(student.getId())) {
                    markService.buildFormalLibrary(student, group, subject);
                    lastBuildTime = student.getUploadTime();
                    count++;
                }
            } catch (Exception e) {
                log.error(
                        "build formal library error for studentId=" + student.getId() + ", groupNumber="
                                + group.getNumber(), e);
            } finally {
                lockService.unwatch(LockType.STUDENT, student.getId());
                lockService.unwatch(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            }
            // 取下一个考生
            student = studentService.findUnLibraryStudent(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    lastBuildTime);
        }
        // 有新任务创建，同时正评状态，此时才需要更新任务数量
        if (count > 0) {
            group = groupService.findOne(group.getExamId(), group.getSubjectCode(), group.getNumber());
            if (group.getStatus() == MarkStatus.FORMAL) {
                markService.updateLibraryCount(group);
                markService.updateMarkedCount(group);
            }
        }
    }

    private void buildTrialLibrary(long trialCount, MarkGroup group) {
        while (trialCount > group.getLibraryCount()) {
            // 取一个未生成试评任务的考生
            ExamStudent student = studentService.randomUnTrialStudent(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            if (student == null) {
                return;
            }
            // 尝试构造试评任务
            try {
                lockService.watch(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
                lockService.watch(LockType.STUDENT, student.getId());
                // 上锁后重新验证分组状态
                if (!groupService.validateStatus(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                        MarkStatus.TRIAL)) {
                    break;
                }
                // 上锁后重复验证考生状态
                if (studentService.validateStatus(student.getId())) {
                    markService.buildTrialLibrary(student, group);
                    markService.updateLibraryCount(group);
                    markService.updateMarkedCount(group);
                }
            } catch (Exception e) {
                log.error(
                        "build trial library error for studentId=" + student.getId() + ", groupNumber="
                                + group.getNumber(), e);
            } finally {
                lockService.unwatch(LockType.STUDENT, student.getId());
                lockService.unwatch(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            }
        }
    }

    @Scheduled(fixedDelay = 30 * 60 * 1000, initialDelay = 2 * 60 * 1000)
    public void updateReport() {
        log.info("start auto-update report");
        try {
            Map<Integer, Set<String>> map = new HashMap<Integer, Set<String>>();
            List<Integer> examIds = groupService.findExamIdByStatus(MarkStatus.FORMAL);
            for (Integer examId : examIds) {
                List<ExamSubject> list = subjectService.list(examId);
                Set<String> set = new HashSet<>();
                for (ExamSubject subject : list) {
                    List<MarkGroup> groups = groupService.findByExamAndSubjectAndStatus(subject.getExamId(),
                            subject.getCode(), MarkStatus.TRIAL);
                    if (!groups.isEmpty()
                            && !lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                        set.add(subject.getCode());
                    }
                }
                if (!set.isEmpty()) {
                    map.put(examId, set);
                }
            }
            for (Integer examId : map.keySet()) {
                subjectiveMap = new HashMap<String, List<ExamQuestion>>();
                objectiveMap = new HashMap<String, List<ExamQuestion>>();
                subjectMap = new HashMap<String, ExamSubject>();
                reportExam(examId, map.get(examId));
            }
        } catch (Exception e) {
            log.error("auto-update report error", e);
        } finally {
            log.info("finish auto-update report");
        }
    }

    private void reportExam(int examId, Set<String> subjectCodeSet) {
        Exam exam = examService.findById(examId);
        if (exam == null || exam.getStatus() != ExamStatus.START) {
            log.info("report exception for examId=" + examId + ", exam is null or status error");
            return;
        }
        if (subjectCodeSet == null) {
            subjectCodeSet = new HashSet<String>();
            List<ExamSubject> list = subjectService.list(exam.getId());
            for (ExamSubject subject : list) {
                List<MarkGroup> groups = groupService.findByExamAndSubjectAndStatus(examId, subject.getCode(),
                        MarkStatus.FORMAL, MarkStatus.TRIAL);
                if (groups != null && !groups.isEmpty()
                        && !lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                    subjectCodeSet.add(subject.getCode());
                }
            }
        }
        log.info("start report for examId=" + examId + ", subjectCode count=" + subjectCodeSet.size());

        lockService.trylock(LockType.SCORE_CALCULATE, examId);
        for (String subjectCode : subjectCodeSet) {
            // 尝试上锁，失败直接跳过
            if (!lockService.trylock(LockType.SCORE_CALCULATE, examId, subjectCode)) {
                log.warn("report locked for examId=" + examId + ", subjectCode=" + subjectCode);
                continue;
            }
            try {
                log.info("start report for examId=" + examId + ", subjectCode=" + subjectCode);
                // 删除原有统计数据
                reportService.deleteData(examId, subjectCode);
                context = new ReportContext(exam);
                int pageNumber = 1;
                int pageSize = 1000;
                List<ExamStudent> list = studentService.findByExamIdAndSubjectCode(examId, subjectCode, pageNumber,
                        pageSize);
                while (list != null && list.size() > 0) {
                    for (ExamStudent student : list) {
                        // 统计
                        statistic(student);
                    }
                    pageNumber++;
                    list = studentService.findByExamIdAndSubjectCode(examId, subjectCode, pageNumber, pageSize);
                }
                // 结束统计
                context.save();
                // 生成文件
                reoprtExcel(examId, subjectCode);
            } catch (Exception e) {
                log.error("report exception for examId=" + examId + ", subjectCode=" + subjectCode, e);
            } finally {
                lockService.unlock(LockType.SCORE_CALCULATE, examId, subjectCode);
                log.info("finish report for examId=" + examId + ", subjectCode=" + subjectCode);
            }
        }
        lockService.unlock(LockType.SCORE_CALCULATE, examId);
    }

    private void statistic(ExamStudent student) {
        if (SubjectiveStatus.MARKED.equals(student.getSubjectiveStatus())
                || SubjectiveStatus.INSPECTED.equals(student.getSubjectiveStatus())) {
            student.setSubject(findExamSubject(student.getExamId(), student.getSubjectCode()));
            student.setObjectiveQuestionList(findQuestionList(student.getExamId(), student.getSubjectCode(),
                    student.getPaperType(), true));
            student.setSubjectiveQuestionList(findQuestionList(student.getExamId(), student.getSubjectCode(),
                    student.getPaperType(), false));
            context.process(student);
        }
    }

    private List<ExamQuestion> findQuestionList(int examId, String subjectCode, String paperType, boolean objective) {
        if (objective) {
            String key = subjectCode + "_" + StringUtils.trimToEmpty(paperType);
            List<ExamQuestion> list = objectiveMap.get(key);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjectiveAndPaperType(examId, subjectCode, true,
                        paperType);
                objectiveMap.put(key, list);
            }
            return list;
        } else {
            List<ExamQuestion> list = subjectiveMap.get(subjectCode);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, false);
                subjectiveMap.put(subjectCode, list);
            }
            return list;
        }
    }

    private ExamSubject findExamSubject(int examId, String subjectCode) {
        ExamSubject subject = subjectMap.get(subjectCode);
        if (subject == null) {
            subject = subjectService.find(examId, subjectCode);
            subjectMap.put(subjectCode, subject);
        }
        return subject;
    }

    private void reoprtExcel(int examId, String subjectCode) throws IOException, Exception {
        uploadRange(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), "课程分段统计");
        uploadCollege(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), "学院分析");
        uploadClass(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), "班级分析");
        uploadTeacher(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), "任课老师统计");
        uploadQuestion(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), true, "客观题分析");
        uploadQuestion(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), false, "主观题分析");
        uploadGroup(examId, subjectCode, findExamSubject(examId, subjectCode).getName(), "分组统计分析");
    }

    private void uploadGroup(int examId, String subjectCode, String subjectName, String name) throws IOException,
            Exception {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectGroup> list = reportSubjectGroupService.findByQuery(query);
        for (ReportSubjectGroup r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setScoreRate(new BigDecimal(r.getScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setStdev(new BigDecimal(r.getStdev()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setCoefficient(new BigDecimal(r.getCoefficient()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectGroup.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private void uploadQuestion(int examId, String subjectCode, String subjectName, boolean objective, String name)
            throws IOException, Exception {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setObjective(objective);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectQuestion> list = reportSubjectQuestionService.findByQuery(query);
        for (ReportSubjectQuestion r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setStdev(new BigDecimal(r.getStdev()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setScoreRate(new BigDecimal(r.getScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setFullScoreRate(new BigDecimal(r.getFullScoreRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectQuestion.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private void uploadTeacher(int examId, String subjectCode, String subjectName, String name) throws IOException,
            Exception {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectTeacher> list = reportSubjectTeacherService.findByQuery(query);
        for (ReportSubjectTeacher r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setExcellentRate(new BigDecimal(r.getExcellentRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setPassRate(new BigDecimal(r.getPassRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setRelativeAvgScore(new BigDecimal(r.getRelativeAvgScore()).setScale(2, RoundingMode.HALF_UP)
                    .doubleValue());
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectTeacher.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private void uploadClass(int examId, String subjectCode, String subjectName, String name) throws IOException,
            Exception {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectClass> list = reportSubjectClassService.findByQuery(query);
        for (ReportSubjectClass r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setExcellentRate(new BigDecimal(r.getExcellentRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setPassRate(new BigDecimal(r.getPassRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectClass.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private void uploadCollege(int examId, String subjectCode, String subjectName, String name) throws IOException,
            Exception {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        List<ReportSubjectCollege> list = reportSubjectCollegeService.findByQuery(query);
        for (ReportSubjectCollege r : list) {
            r.setAvgScore(new BigDecimal(r.getAvgScore()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setExcellentRate(new BigDecimal(r.getExcellentRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
            r.setPassRate(new BigDecimal(r.getPassRate()).setScale(2, RoundingMode.HALF_UP).doubleValue());
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectCollege.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private void uploadRange(int examId, String subjectCode, String subjectName, String name) throws IOException,
            Exception {
        List<ReportSubjectRangeDTO> list = new ArrayList<ReportSubjectRangeDTO>();
        ReportSubject subject = reportSubjectService.findOne(examId, subjectCode);
        if (subject != null) {
            JSONArray array = getScoreRange(subject.getScoreRange(), subject.getTotalScore(),
                    subject.getRealityCount(), 10);
            for (int i = 0; i < array.size(); i++) {
                JSONObject jsonObject = array.getJSONObject(i);
                String score = jsonObject.getInt("score") + "-";
                Integer rangeCount = jsonObject.getInt("rangeCount");
                Double rangeRate = jsonObject.getDouble("rangeRate");
                list.add(new ReportSubjectRangeDTO(score, rangeCount, rangeRate));
            }
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        new ExportExcel(name, ReportSubjectRangeDTO.class).setDataList(list).write(os);
        os.flush();
        byte[] value = os.toByteArray();
        String md5 = CodecUtils.toHexString(CodecUtils.md5(value));
        fileService.uploadReport(new ByteArrayInputStream(value), md5, examId, subjectCode + "-" + subjectName, name,
                FormatType.XLSX);
    }

    private JSONArray getScoreRange(String scoreRange, double totalScore, Integer totalCount, int range) {
        JSONArray result = new JSONArray();
        JSONObject jsonObject = JSONObject.fromObject(scoreRange);
        int rangeCount = 0;
        int sumCount = 0;
        int total = (int) Math.ceil(totalScore / range);
        for (int i = total; i >= 0; i--) {
            int start = i * range;
            int end = (i - 1) * range + 1;
            if (start > totalScore) {
                start = (int) Math.ceil(totalScore);
            }
            if (end < 0) {
                end = 0;
            }
            rangeCount = getSumCount(jsonObject, start, end);
            sumCount = sumCount + rangeCount;
            result.add(getRangeJson(totalCount, rangeCount, sumCount, i * range));
        }
        return result;
    }

    private int getSumCount(JSONObject jsonObject, int start, int end) {
        int sumCount = 0;
        int currentCount = 0;
        if (start < end) {
            for (int i = start; i <= end; i++) {
                currentCount = jsonObject.getInt(String.valueOf(i));
                sumCount = sumCount + currentCount;
            }
        } else if (start >= end) {
            for (int i = end; i <= start; i++) {
                currentCount = jsonObject.getInt(String.valueOf(i));
                sumCount = sumCount + currentCount;
            }
        } else {
            sumCount = jsonObject.getInt(String.valueOf(start));
        }
        return sumCount;
    }

    private JSONObject getRangeJson(Integer totalCount, int rangeCount, int sumCount, int score) {
        JSONObject value = new JSONObject();
        value.accumulate("score", score);
        value.accumulate("rangeCount", rangeCount);
        value.accumulate("rangeRate", rangeCount * 100.0 / totalCount);
        value.accumulate("sumCount", sumCount);
        value.accumulate("sumRate", sumCount * 100.0 / totalCount);
        return value;
    }
}
